@@ -8,6 +8,8 @@ def pipe(context):
     '''
     @pipe({
         'model': 'bookstore.BookInstance',
+        # 'method': 'create', # insert mode as default
+        # 'method': 'update', # update mode
         'foreignkey_fields':{
             'book':{
                 'model': 'bookstore.Book',
@@ -43,7 +45,8 @@ def pipe(context):
         def pipe(*args, **kwargs):
             results = parser(*args, **kwargs)
             reducer = PipeReducer(context, results)
-            reducer.reduce()
+            if reducer.is_valid():
+                reducer.reduce()
             return results
         return pipe
     return wrapper
