@@ -1,6 +1,7 @@
+import inspect, itertools, os
+
 from django.apps import apps
 from django.db import models
-from django.db.models import Model
 
 from .vars import *
 
@@ -27,7 +28,7 @@ def get(model, unique_key, item):
 
 
 def update(model, unique_key, item):
-    object = get(model, unique_key, item)
+    # object = get(model, unique_key, item)
     unique_keys = _validate_uniquekey(unique_key)
     
     pkset = {k:v for k, v in item.items() if k in unique_keys}
@@ -37,10 +38,6 @@ def update(model, unique_key, item):
     return obj
 
 
-
-
-
-
 def create(model, item):
     return model.objects.create(**item)
 
@@ -48,7 +45,7 @@ def create(model, item):
 def get_model(model_name):
     if isinstance(model_name, str):
         model = apps.get_model(model_name)
-    elif issubclass(model_name.__class__, Model):
+    elif issubclass(model_name, models.Model):
         model = model_name
     else:
         raise ValueError(f"The type of {model_name} must be str or model class")
@@ -59,6 +56,7 @@ def pop_m2m_column(item, m2m_fields):
     if m2m_fields:
         m2mset = {
             field:item.pop(field) for field in m2m_fields
+            if field in item
         }
         return m2mset
 
@@ -132,6 +130,62 @@ def get_or_create(
     return object
 
     
+def module2dict(module):
+    return dict(itertools.takewhile(lambda i: i[0] != '__builtins__', inspect.getmembers(module)))
+
+
+def get_base_module_name(module):
+    *_, base = module.__name__.split('.')
+    return base
 
 
 
+def get_module_dir(module):
+    return os.path.dirname(module.__path__[0])
+
+
+
+from glob import glob
+import os
+
+
+def is_subdir(root_dir, sub_dir):
+    if os.path.isfile(root_dir):
+        root_dir = os.path.dirname(root_dir)
+    root = os.path.abspath(root_dir)
+    sub = os.path.abspath(sub_dir)
+
+    if len(root) != len(sub):
+        return root == sub[:len(root)]
+    return root != sub
+
+
+def find_models_module(current=None):
+    if not current:
+        stack, *_ = [
+            stack for stack in inspect.stack()
+            if not is_subdir(__file__, stack.filename)
+        ]
+        current = stack.filename
+
+    if os.path.isfile(current):
+        current = os.path.dirname(current)
+
+    root = os.path.abspath('.')
+
+    paths = []
+    while is_subdir(root, current):
+        for path in glob('**/models.py', recursive=True, root_dir=current):
+            p = os.path.join(current, path)
+            if p in paths:
+                continue
+            paths.append(p)
+        current = os.path.dirname(current)
+
+    if not paths:
+        raise ValueError('Cannot find package which contains models.py')
+
+    if len(paths) > 1:
+        raise ValueError(f'Muitiple module path matched: {paths}')
+
+    return os.path.dirname(paths[0])
